/*
 * Copyright 2020 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

#include <descrip.h>
#include <ssdef.h>

#include "sha256.h"

#include "tim.h"
#include "syi.h"
#include "dvi.h"
#include "jpi.h"
#include "stk.h"
#include "cnt.h"
#include "hsh.h"

#include "extran.h"

long int get_extran_data(char *buf, long int flag)
{
    char tmp[32768];
    int tmplen = 0;
    $DESCRIPTOR(sysdsk, "SYS$SYSDEVICE");
    $DESCRIPTOR(usrdsk, "SYS$DISK");
    SHA256_CTX ctx;
    if(flag & (EXTRAN_SYI + EXTRAN_DVI_SYS + EXTRAN_DVI_USR + EXTRAN_JPI + EXTRAN_STK))
    {
        if(flag & EXTRAN_TIM)
        {
            tmplen += get_tim_data(tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_SYI)
        {
            tmplen += get_syi_data(tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_DVI_SYS)
        {
            tmplen += get_dvi_data(&sysdsk, tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_DVI_USR)
        {
            tmplen += get_dvi_data(&usrdsk, tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_JPI)
        {
            tmplen += get_jpi_data(0, tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_STK)
        {
            tmplen += get_stk_data(tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_CNT)
        {
            tmplen += get_cnt_data(tmp + tmplen, sizeof(tmp) - tmplen);
        }
        if(flag & EXTRAN_HSH)
        {
            tmplen += get_hsh_data(tmp, tmplen, tmp + tmplen, sizeof(tmp) - tmplen);
        }
    }
    else
    {
        return SS$_BADPARAM;
    }
    sha256_init(&ctx);
    sha256_update(&ctx, (unsigned char *)tmp, tmplen);
    sha256_final(&ctx, (unsigned char *)buf);
    return SS$_NORMAL;
}

