/*
 * Copyright 2020 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

#ifndef EXTRAN_H
#define EXTRAN_H

#define EXTRAN_TIM 1
#define EXTRAN_SYI 2
#define EXTRAN_DVI_SYS 4
#define EXTRAN_DVI_USR 8
#define EXTRAN_JPI 16
#define EXTRAN_STK 32
#define EXTRAN_CNT 64
#define EXTRAN_HSH 128

/*
 * get_extran_data
 *
 * arguments:
 *     buf - pointer to char array that will receive 32 random bytes with SHA-256 of source data
 *     flags - mask of sources to use
 *                 EXTRAN_TIM 8 bytes from SYS$GETTIM (not unpredictable)
 *                 EXTRAN_SYI 112 bytes from SYS$GETSYIW
 *                 EXTRAN_DVI_SYS 64 bytes from SYS$GETDVIW on SYS$SYSDEVIVE
 *                 EXTRAN_DVI_USR 64 bytes from SYS$GETDVIW on SYS$DISK
 *                 EXTRAN_JPI 520 bytes from SYS$GETJPIW
 *                 EXTRAN_STK 8192 bytes from stack
 *                 EXTRAN_CNT 40 bytes from an array of counters (not unpredictable but ensures change with multiple calls)
 *                 EXTRAN_HSH 32 bytes of SHA-256 of all other data (not unpredictable but may make reversing more difficult)
 *
 * return value:
 *     SS$_NORMAL - 32 random bytes generated
 *     SS$_BADPARAM - non valid flags resulting in no random bytes generated
 *
 */
long int get_extran_data(char *buf, long int flag);

#endif
